__title__ = "officialeye"
__description__ = "An AI-powered generic document-analysis tool."
__url__ = "https://officialeye.zerobone.net"
__version__ = "1.1.5"
__author__ = "Alexander Mayorov"
__author_email__ = "zb@zerobone.net"
__license__ = "GPL-3.0"
__copyright__ = "Copyright Alexander Mayorov"
